#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <string.h>
#include "com.cpp"
#include "linpack.cpp"
#include "ranlib0123.cpp"
#include "ranlib.h"
using namespace std;

#define PI 3.1415926

int geneN;     // numbers of genes
int geneM;     // numbers of chips
int LOOP;         // default LOOP=50  
int NUMBER;       // default NUMBER=10  
double cutoffx;   // default cutoffx=100.0
double cutoffp;   // default cutoffp=2.0
double cutoffa;   // default cutoffa=0.5
int fixrow;    // fix the first "fixrow" genes as target                        default fixrow=0
int fixcol;    // fix the first "fixcol" experimental conditions as foreground  default fixcol=0

double *alpha0;
double *beta0;
double *alpha1;
double *beta1;
double *tau0;

double *r;
double *rtemp;
double *rtemp1;
double *e;
double *etemp;
double *etemp1;
double *a;
double *atemp;
double *atemp1;

double ** c;
double ** ctemp;

double *priorrow;
double *priorcol;
double **priorc;

double ** loglike;
double ** z;

double truelike;
int    logid;

double power(double x, double y) // x^y
{
 return( exp( y*log(x) ) );
}

double gammln(double xx)
{
	double x,y,tmp,ser;
	static double cof[6]={76.18009172947146,-86.50532032941677,
	    24.01409824083091,-1.231739572450155,
	    0.1208650973866179e-2,-0.5395239384953e-5};
	int j;

	y=x=xx;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.000000000190015;
	for (j=0;j<=5;j++) ser += cof[j]/++y;
	return ( -tmp+log(2.5066282746310005*ser/x) );
}

void setlabel(double *x)
{
  int i,j;
  double indicate;

  for(i=0;i<geneN;i++)
  {indicate=genunf(0,1);                // randomly assign 0/1 to each row
   if(indicate<0.5) rtemp[i]=1.0;
   else if(indicate>=0.5) rtemp[i]=0.0;
  }

  for(j=0;j<geneM;j++)
  {
	if(fabs(x[j])>cutoffx) {etemp[j]=1.0;} //if fabs(x[j])>cutoffx, force this column into target column 
	
	else if(fabs(x[j])<=cutoffx)
	{indicate=genunf(0,1);                 //randomly assign 0/1 to each column
     if(indicate<0.5) etemp[j]=1.0;
     else if(indicate>0.5) etemp[j]=0.0;
	}
  }
  return;
}

void updatea(double *x, double **y) // least square to estimate a_hat
{
  int i,j;
  double s,t,u,v,cut;

  for(i=0;i<geneN;i++)
  {
    s=0.0;
    t=0.0;
    u=0.0;
	v=0.0;
	for(j=0;j<geneM;j++) 
    {s=s+x[j]*y[i][j]*etemp[j];
     t=t+x[j]*x[j]*etemp[j];
	 u=u+etemp[j];}
     
	 atemp[i]=s/t;          //estimator of a_hat
    
	for(j=0;j<geneM;j++)
	{v=v+ (y[i][j]-x[j]*atemp[i])*(y[i][j]-x[j]*atemp[i])*etemp[j]; }
	 v=v/(u-1.0);

	 cut=atemp[i] / (sqrt(v/t)); //test significant of a_hat

	 if(fabs(cut)>=1.96 && fabs(atemp[i])>=cutoffa) {atemp[i]=s/t;}
	 else if(fabs(cut)<1.96 || fabs(atemp[i]<cutoffa)) {atemp[i]=1.0;}
  }
  return;
}


double post(double **z, double *rtemp, double *etemp, double **ctemp)
{
  int i,j;
  double A[geneM], B[geneM], C[geneM], sumzr[geneM], sumr[geneM];

  for(j=0;j<geneM;j++)
  {
     A[j]=0.0;
	 B[j]=0.0;
	 C[j]=0.0;
	 sumzr[j]=0.0;
	 sumr[j]=0.0;
     for(i=0;i<geneN;i++)
	 {
	     A[j]=A[j]+z[i][j]*z[i][j]*(1.0-rtemp[i]*etemp[j]*ctemp[i][j]);
		 B[j]=B[j]+z[i][j]*(1.0-rtemp[i]*etemp[j]*ctemp[i][j]);
		 C[j]=C[j]+1.0-rtemp[i]*etemp[j]*ctemp[i][j];
		 sumzr[j]=sumzr[j]+z[i][j]*z[i][j]*rtemp[i]*ctemp[i][j];
		 sumr[j]=sumr[j]+rtemp[i]*ctemp[i][j];
	 }
  }

  double a;
  a=0.0;
  for(j=0;j<geneM;j++)
  {
    a=a+ gammln(alpha1[j]+0.5*etemp[j]*sumr[j]) 
	  - (alpha1[j]+0.5*etemp[j]*sumr[j])*log(beta1[j]+0.5*etemp[j]*sumzr[j])
	  - 0.5*log(1+C[j]) 
	  + gammln(alpha0[j]+0.5*C[j])
	  - (alpha0[j]+0.5*C[j])*log(0.5*A[j]+0.5*tau0[j]*tau0[j]+beta0[j]-0.5*(B[j]+tau0[j])*(B[j]+tau0[j])/(1.0+C[j]));
  }
 
  double b;
  b=0.0;
  for(i=0;i<geneN;i++)
  {
    b=b+rtemp[i]*log( priorrow[i]/(1.0-priorrow[i]) );
  }

  double c;
  c=0.0;
  for(j=0;j<geneM;j++)
  {
     c=c+etemp[j]*log( priorcol[j]/(1.0-priorcol[j]) );
  }

  double d;
  d=0.0;
  for(i=0;i<geneN;i++)
  {
     for(j=0;j<geneM;j++)
	 {
	   d=d+ctemp[i][j]*log( priorc[i][j]/(1.0-priorc[i][j]) );
	 }
  }
  return(a+b+c+d);
}

void est(double *x, double **y) //estmate tau_0j alpha_0j beta_0j alpha_1j beta_1j
{
  int i,j;
  double sum1,sum2,sumtmp,v,cut;

  double tempz[geneN][geneM];
  double tempa[geneN];

  for(i=0;i<geneN;i++)
  {
      sum1=0.0;
      sum2=0.0;
      v=0.0;
	  for(j=0;j<geneM;j++)
	  {
	    sum1=sum1+x[j]*y[i][j];
		sum2=sum2+x[j]*x[j];
	  }

	  tempa[i]=sum1/sum2;

	 for(j=0;j<geneM;j++) {v=v+ (y[i][j]-x[j]*tempa[i])*(y[i][j]-x[j]*tempa[i]); }
	 v=v/(geneM-1.0);

	 cut=tempa[i] / (sqrt(v/sum2));

	 if(fabs(cut)>=1.96 && fabs(tempa[i])>=cutoffa) {tempa[i]=sum1/sum2;}
	 else if(fabs(cut)<1.96 || fabs(tempa[i])<cutoffa) {tempa[i]=1.0;}
  }                                              //estimate linear factor a_i using all columns

  for(i=0;i<geneN;i++)
  {
     for(j=0;j<geneM;j++)
	 {
	    tempz[i][j]=y[i][j]-x[j]*tempa[i];
	 }
  }
  
  for(j=0;j<geneM;j++)
  {
    sumtmp=0.0;
	for(i=0;i<geneN;i++) {sumtmp=sumtmp+tempz[i][j];}
	tau0[j]=sumtmp/geneN;
  }                              //tau_0j: column mean

  double var[geneM];
  for(j=0;j<geneM;j++)
  {
    sum1=0.0;
    sum2=0.0;
    for(i=0;i<geneN;i++)
	{
	   sum1=sum1+tempz[i][j];
	   sum2=sum2+tempz[i][j]*tempz[i][j];
	}

	var[j]=(sum2-sum1*sum1/geneN)/geneN;
  }                             //get the variance of each column

  sum1=0;
  sum2=0;
  double varsigma;
  for(j=0;j<geneM;j++)
  {
    sum1=sum1+var[j];
	sum2=sum2+var[j]*var[j];
  }

  varsigma=(sum2-sum1*sum1/geneM)/geneM;  // variance of sigma_0j
 
  for(j=0;j<geneM;j++)
  {
     alpha0[j]=var[j]*var[j] / varsigma +2.0;
	 beta0[j]=(alpha0[j]-1.0) * var[j];
  }                             //Inverse Gamma(alpha_0j,beta_0j) mean:column variance, var:sample variance
  
  double cutvalue;
  cutvalue=cutoffp;

  double cutoff[geneM];
  for(j=0;j<geneM;j++){ cutoff[j]=sqrt(var[j])/cutvalue; } //treat large diffence as noise

  double colvar[geneM];
  double countsignal;
  for(j=0;j<geneM;j++)
  {
    sum1=0.0;
	sum2=0.0;
	countsignal=0.0;
	for(i=0;i<geneN;i++)
	{
	   if(fabs(tempz[i][j])<=cutoff[j])
	   {
	    sum1=sum1+tempz[i][j];
		sum2=sum2+tempz[i][j]*tempz[i][j];
		countsignal=countsignal+1.0;
	   }
	}
	colvar[j]=(sum2-sum1*sum1/countsignal)/countsignal;
  }                  //column variance based on reduced dataset
  
  sum1=0.0;
  sum2=0.0;
  countsignal=0.0;
  for(j=0;j<geneM;j++)
  { 
	 if(colvar[j]>0.000001)
	 {
     sum1=sum1+colvar[j];
	 sum2=sum2+colvar[j]*colvar[j];
	 countsignal=countsignal+1.0;
	 }
  }
  double vartmp;
  vartmp=(sum2-sum1*sum1/countsignal)/countsignal;

  for(j=0;j<geneM;j++)
  {
     alpha1[j]=colvar[j]*colvar[j] / vartmp +2.0;
	 beta1[j]=(alpha1[j]-1.0) * colvar[j];
  }        //Inverse Gamma(alpha_1j,beta_1j) mean: reduced column variance, var:reduced sample variance
}

void updatec(double *x, double **y)
{
  int i, j, loop, n;
  loop=LOOP;
  double p1,p0,like,liketemp,indicate,cutoff;

  for(i=0;i<geneN;i++)
  {
     for(j=0;j<geneM;j++)
	 {
	   if(e[j]==1.0) z[i][j]=y[i][j]-x[j]*a[i];
	   else if(e[j]==0.0) z[i][j]=y[i][j]-x[j];
	 }
  }
   
  for(n=0;n<loop;n++)
  {
  if(n==0) {liketemp=post(z,r,e,ctemp);}
  
  if(n<=loop*0.9)
  {
  for(i=0;i<geneN;i++)
  {
     for(j=0;j<geneM;j++)
	 {
	    if(r[i]==1.0 && e[j]==1.0)
		{
		  ctemp[i][j]=1.0;
	      p1=post(z,r,e,ctemp);
			 
	      ctemp[i][j]=0.0;
	      p0=post(z,r,e,ctemp);

		  indicate=genunf(0,1);
          cutoff=exp(p1-p0)/(exp(p1-p0)+1.0);
	  
          if(indicate<=cutoff) ctemp[i][j]=1.0;
	      else if(indicate>cutoff) ctemp[i][j]=0.0;
		}
	 }
  }
  }
  
  else if(n>loop*0.9)
  {
  for(i=0;i<geneN;i++)
  {
     for(j=0;j<geneM;j++)
	 {
	    if(r[i]==1.0 && e[j]==1.0)
		{
		  ctemp[i][j]=1.0;
	      p1=post(z,r,e,ctemp);
			 
	      ctemp[i][j]=0.0;
	      p0=post(z,r,e,ctemp);
          
          if(p1>=p0) ctemp[i][j]=1.0;
	      else if(p1<p0) ctemp[i][j]=0.0;
		}
	 }
  }
  }

  like=post(z,r,e,ctemp);

  if(like>liketemp) 
  {   liketemp=like; 	  
      for(i=0;i<geneN;i++)
	  {for(j=0;j<geneM;j++)
	  {c[i][j]=ctemp[i][j];}
	  }
  }
  }
}

double update(double *x, double **y)
{
  int i, j, k, loop;
  loop=LOOP;
  double p1, p0, like, liketemp, indicate, cutoff;

  setlabel(x);

  for(i=0;i<geneN;i++) {rtemp1[i]=rtemp[i];}
  for(j=0;j<geneM;j++) {etemp1[j]=etemp[j];}

  for(k=0;k<loop;k++)
  {

  updatea(x,y);

  for(i=0;i<geneN;i++)
  {
     for(j=0;j<geneM;j++)
	 {
	   if(etemp[j]==1.0) z[i][j]=y[i][j]-x[j]*atemp[i];
	   else if(etemp[j]==0.0) z[i][j]=y[i][j]-x[j];
	 }
  }

  if(k==0) {liketemp=post(z, rtemp, etemp, ctemp);}
  
  if(k<=loop*0.9)
  {
  for(i=0;i<fixrow;i++){rtemp[i]=1.0;}

  for(i=fixrow;i<geneN;i++)
  {
     rtemp[i]=1.0;
	 p1=post(z, rtemp, etemp, ctemp);

	 rtemp[i]=0.0;
	 p0=post(z, rtemp, etemp, ctemp);

	 indicate=genunf(0,1);
     cutoff=exp(p1-p0)/(exp(p1-p0)+1.0);
	  
     if(indicate<=cutoff) rtemp[i]=1.0;
	 else if(indicate>cutoff) rtemp[i]=0.0;
  }
   
  for(j=0;j<fixcol;j++){etemp[j]=1.0;}
  
  for(j=fixcol;j<geneM && fabs(x[j])<=cutoffx;j++)
  {
      etemp[j]=1.0;
	  p1=post(z, rtemp, etemp, ctemp);

	  etemp[j]=0.0;
	  p0=post(z, rtemp, etemp, ctemp);

	  indicate=genunf(0,1);
      cutoff=exp(p1-p0)/(exp(p1-p0)+1.0);

	  if(indicate<=cutoff) etemp[j]=1.0;
	  else if(indicate>cutoff) etemp[j]=0.0;	  
  }
  }

  else if(k>loop*0.9)
  {
  for(i=0;i<fixrow;i++){rtemp[i]=1.0;}

  for(i=fixrow;i<geneN;i++)
  {
     rtemp[i]=1.0;
	 p1=post(z, rtemp, etemp, ctemp);

	 rtemp[i]=0.0;
	 p0=post(z, rtemp, etemp, ctemp);
	  
     if(p1>=p0) rtemp[i]=1.0;
	 else if(p1<p0) rtemp[i]=0.0;
  }

  for(j=0;j<fixcol;j++){etemp[j]=1.0;}

  for(j=fixcol;j<geneM && fabs(x[j])<=cutoffx;j++)
  {
      etemp[j]=1.0;
	  p1=post(z, rtemp, etemp, ctemp);

	  etemp[j]=0.0;
	  p0=post(z, rtemp, etemp, ctemp);

	  if(p1>=p0) etemp[j]=1.0;
	  else if(p1<p0) etemp[j]=0.0;	  
  }
  }

  like=post(z, rtemp, etemp, ctemp);

  loglike[k][logid]=like; //keep the log-likelihood in each step
  
  if(like>liketemp) 
  { 
	  liketemp=like; 

	  for(i=0;i<geneN;i++) {rtemp1[i]=rtemp[i];}
      for(j=0;j<geneM;j++) {etemp1[j]=etemp[j];}
	  for(i=0;i<geneN;i++) {atemp1[i]=atemp[i];}
  }
  }
  return(liketemp);
}

void gibbssampler(double *x, double **y)
{	
  int i,j;

    for(i=0;i<geneN;i++)
  {
    for(j=0;j<geneM;j++)
	{
	  priorc[i][j]=0.5;
	  c[i][j]=1.0;
	  ctemp[i][j]=1.0;
	}
  }

    for(i=0;i<geneN;i++){priorrow[i]=0.5;}
	for(j=0;j<geneM;j++){priorcol[j]=0.5;}

  FILE *s;
/* begin gibbs sampler */
  est(x,y);

  double like,liketemp;
  int k, loop;
  loop=NUMBER; //number of initial point 

  for(k=0;k<loop;k++)
  {
   logid=k;  //keep log-likelihood for each parallel chain
 	  
   like=update(x,y);
   
   if(k==0) liketemp=like;

   if(like>=liketemp)
   {liketemp=like;
    for(i=0;i<geneN;i++) r[i]=rtemp1[i];
	for(j=0;j<geneM;j++) e[j]=etemp1[j];
	for(i=0;i<geneN;i++) a[i]=atemp1[i];
   }
   
   printf("The %d th sampler\n", k+1);
  }

  for(i=0;i<geneN;i++)
  {
     for(j=0;j<geneM;j++)
	 {
	   if(e[j]==1.0) z[i][j]=y[i][j]-x[j]*a[i];
	   else if(e[j]==0.0) z[i][j]=y[i][j]-x[j];
	 }
  }
 
   truelike=post(z,r,e,c);
/* end gibbs sampler */

/* estimate cell-level noise */
  updatec(x, y);

  s=fopen("cellnoise.txt","w");
  for(i=0;i<geneN;i++)
  {
     for(j=0;j<geneM;j++)
	 {
	    if(c[i][j]==1.0) fprintf(s, "%d ", 1);
		else if(c[i][j]==0.0) fprintf(s, "%d ", 0);
	 }
	fprintf(s, "\n");
  }
  fclose(s);

  for(i=0;i<geneN;i++)
  {
     for(j=0;j<geneM;j++)
	 {
	    ctemp[i][j]=c[i][j];
	 }
  }

  liketemp=truelike;

  for(k=0;k<loop;k++)
  {
   logid=k+NUMBER;
	  
   like=update(x,y);
   
   if(like>=liketemp)
   {liketemp=like;
    for(i=0;i<geneN;i++) r[i]=rtemp1[i];
	for(j=0;j<geneM;j++) e[j]=etemp1[j];
	for(i=0;i<geneN;i++) a[i]=atemp1[i];
   }

   printf("The %d th sampler (cell level) \n", k+1);
  }

  for(i=0;i<geneN;i++)
  {
     for(j=0;j<geneM;j++)
	 {
	   if(e[j]==1.0) z[i][j]=y[i][j]-x[j]*a[i];
	   else if(e[j]==0.0) z[i][j]=y[i][j]-x[j];
	 }
  }

/* rowpost & colpost: 
   col0: number i
   col1: r[i] & e[j]
   col2: posterior probability
   col3: raw difference */
  double rowpost[geneN][4];
  double colpost[geneM][4];
  
  for(i=0;i<geneN;i++)
  {
    rowpost[i][0]=i;
	rowpost[i][1]=r[i];
  }
 
  for(j=0;j<geneM;j++)
  {
    colpost[j][0]=j;
	colpost[j][1]=e[j];
  }
  
  double tempid;
  double temppost1, temppost0;

  for(i=0;i<geneN;i++)
  {
      tempid=r[i];

	  r[i]=1.0;
	  temppost1=post(z,r,e,c);

	  r[i]=0.0;
	  temppost0=post(z,r,e,c);

	  r[i]=tempid;
	  rowpost[i][2] = exp(temppost1-temppost0) / (1.0+ exp(temppost1-temppost0));
	  rowpost[i][3] = temppost1-temppost0;
  }
  
  for(j=0;j<geneM;j++)
  {
      tempid=e[j];

	  e[j]=1.0;
	  temppost1=post(z,r,e,c);

	  e[j]=0.0;
	  temppost0=post(z,r,e,c);

	  e[j]=tempid;
	  colpost[j][2] = exp(temppost1-temppost0) / (1.0+ exp(temppost1-temppost0));
	  colpost[j][3] = temppost1-temppost0;
  }

/* sort list according to raw difference */
  double t;
  for(i=0;i<geneN-1;i++)
  {
     for(j=0;j<geneN-1-i;j++)
		 if(rowpost[j][3]<rowpost[j+1][3])
	 {
	     t=rowpost[j][0];rowpost[j][0]=rowpost[j+1][0];rowpost[j+1][0]=t;
		 t=rowpost[j][1];rowpost[j][1]=rowpost[j+1][1];rowpost[j+1][1]=t;
		 t=rowpost[j][2];rowpost[j][2]=rowpost[j+1][2];rowpost[j+1][2]=t;
		 t=rowpost[j][3];rowpost[j][3]=rowpost[j+1][3];rowpost[j+1][3]=t;
	 }
  }

  for(i=0;i<geneM-1;i++)
  {
     for(j=0;j<geneM-1-i;j++)
		 if(colpost[j][3]<colpost[j+1][3])
	 {
	     t=colpost[j][0];colpost[j][0]=colpost[j+1][0];colpost[j+1][0]=t;
		 t=colpost[j][1];colpost[j][1]=colpost[j+1][1];colpost[j+1][1]=t;
		 t=colpost[j][2];colpost[j][2]=colpost[j+1][2];colpost[j+1][2]=t;
		 t=colpost[j][3];colpost[j][3]=colpost[j+1][3];colpost[j+1][3]=t;
	 }
  }

/* write "rowpost.txt", "colpost.txt" */
  s=fopen("rowpost.txt","w");
  for(i=0;i<geneN;i++)
  {
    fprintf(s, "%lf ", rowpost[i][0]+1.0);
	fprintf(s, "%lf ", rowpost[i][1]);
	fprintf(s, "%lf ", rowpost[i][2]);
	fprintf(s, "%lf\n", rowpost[i][3]);
  }
  fclose(s);
 
  s=fopen("colpost.txt","w");
  for(j=0;j<geneM;j++)
  {
    fprintf(s, "%lf ", colpost[j][0]+1.0);
	fprintf(s, "%lf ", colpost[j][1]);
	fprintf(s, "%lf ", colpost[j][2]);
	fprintf(s, "%lf\n", colpost[j][3]);
  }
  fclose(s);

  s=fopen("loglike.txt","w");
  for(j=0;j<LOOP;j++)
  {
    for(i=0;i<NUMBER*2;i++) {fprintf(s, "%lf ", loglike[j][i]);}
	fprintf(s, "\n");
  }
  fclose(s);

  s=fopen("factor.txt","w");
  for(i=0;i<geneN;i++)
  {
    fprintf(s, "%lf\n", a[i]);
  }
  fclose(s);
}

  int main(int argc, char **argv)
{ 
  int i,j,b;

  printf("argc=%d\n",argc);

  if(argc<8)
   {
    printf("7 options need to be specified \n\tLOOP,\n\tNUMBER,\n\tcutoffx,\n\tcutoffp,\n\tcutoffa,\n\tfixrow,\n\tfixcol. \n");
    exit(0);
   } 

  	LOOP    = atoi(argv[1]);
	NUMBER  = atoi(argv[2]);
	cutoffx = atof(argv[3]);
	cutoffp = atof(argv[4]);
	cutoffa = atof(argv[5]);
	fixrow  = atoi(argv[6]);
	fixcol  = atoi(argv[7]);
  
  loglike = new double*[LOOP];
  for(i=0;i<LOOP;i++) {	loglike[i] = new double[NUMBER*2]; }

  for(i=0;i<LOOP;i++)
  {
     for(j=0;j<NUMBER*2;j++)
	 {
		 loglike[i][j]=0.0;
	 }
  }
  
///////////////////////////////////////////////
		i=0;
		j=0;

		char * pch=NULL;
        char str[20000];  //need to change str[?] for large dataset

        fstream file_op("database.txt",ios::in);
        while(!file_op.eof() ) 
        {
              file_op.getline(str,20000); //need to change str[?] for large dataset
              
			  pch=strtok(str,"\t");
              while(pch!=NULL)
			  {
				pch=strtok(NULL, "\t");
				j=j+1;
			  }
              
			  j=0;
			  i=i+1;	  

		}         
  		file_op.close();

		geneN=i-1;

		j=0;
        fstream file_op1("query.txt",ios::in);
        while(!file_op1.eof() ) 
        {
              file_op1.getline(str,20000);
              
			  pch=strtok(str,"\t");
              while(pch!=NULL)
			  {
				pch=strtok(NULL, "\t");
				j=j+1;
			  }             			 	  
		}         
  
		file_op1.close();

		geneM=j;
//////////////////////////////////////////////////
 alpha0 = new double[geneM];
 beta0  = new double[geneM];
 alpha1 = new double[geneM];
 beta1  = new double[geneM];
 tau0   = new double[geneM];
 r      = new double[geneN];
 rtemp  = new double[geneN];
 rtemp1 = new double[geneN];
 e      = new double[geneM];
 etemp  = new double[geneM];
 etemp1 = new double[geneM];
 a      = new double[geneN];
 atemp  = new double[geneN];
 atemp1 = new double[geneN];

  z = new double*[geneN];
  for(i=0;i<geneN;i++) { z[i] = new double[geneM]; }

  c = new double*[geneN];
  for(i=0;i<geneN;i++){ c[i] = new double[geneM]; }

  ctemp = new double*[geneN];
  for(i=0;i<geneN;i++){ ctemp[i] = new double[geneM]; }

 priorrow = new double[geneN];
 priorcol = new double[geneM];
  
 priorc = new double*[geneN];
 for(i=0;i<geneN;i++){ priorc[i] = new double[geneM]; }

 double *x;
 double **y;

 x = new double[geneM];

 y = new double*[geneN];
 for(i=0;i<geneN;i++) { y[i] = new double[geneM]; }

/* begin read data: E.coli data set */
    	i=0;
		j=0;

        fstream file_op2("database.txt",ios::in);
        while(!file_op2.eof() ) 
        {
              file_op2.getline(str,20000); //need to change str[?] for large dataset
              
			  pch=strtok(str,"\t");
              while(pch!=NULL)
			  {
			    y[i][j]=atof(pch);
				pch=strtok(NULL, "\t");
				j=j+1;
			  }
              
			  j=0;
			  i=i+1;	  
		}         
  
		file_op2.close();

		j=0;
        fstream file_op3("query.txt",ios::in);
        while(!file_op3.eof() ) 
        {
              file_op3.getline(str,20000);
              
			  pch=strtok(str,"\t");
              while(pch!=NULL)
			  {
			    x[j]=atof(pch);
				pch=strtok(NULL, "\t");
				j=j+1;
			  }             			 	  
		}         
  		file_op3.close();

/* end read data: E.coli data set */

 printf("cutoffx=%lf\n",cutoffx);
  printf("geneN=%d\n",geneN); 
  printf("geneM=%d\n",geneM); 
  printf("\n");

  printf("BEST STARTs! \n");

  gibbssampler(x,y);

  for(i=0;i<LOOP;i++) delete [] loglike[i];
  delete [] loglike;

  for(i=0;i<geneN;i++) delete [] z[i];
  delete [] z;

  delete [] alpha0;
  delete [] beta0;
  delete [] alpha1;
  delete [] beta1;
  delete [] tau0;
  delete [] r;
  delete [] rtemp;
  delete [] rtemp1;
  delete [] e;
  delete [] etemp;
  delete [] etemp1;
  delete [] a;
  delete [] atemp;
  delete [] atemp1;
 
  for(i=0;i<geneN;i++) delete [] c[i];
  delete [] c;
  
  for(i=0;i<geneN;i++) delete [] ctemp[i];
  delete [] ctemp;
 
  delete [] priorrow;
  delete [] priorcol;

  for(i=0;i<geneN;i++) delete [] priorc[i];
  delete [] priorc;

  delete [] x;
  
  for(i=0;i<geneN;i++) delete [] y[i];
  delete [] y;

  FILE *s;  
  s=fopen("end.txt","w");
  fprintf(s, "end!");
  fclose(s);

  printf("BEST ENDs! \n");

  scanf("%d",&b);
		i=b+1;

return (0);
}
